DAT objectName          byte "Max7219MultiArray131226c", 0
CON
{{
  ******* Public Notes *******

  Author: Duane Degn
  Date: December 25, 2013

  MIT License

  An attempt to control an 8x8 LED array with a MAX7219 chip.

  The number of arrays may be changed by changing the
  "ARRAYS_IN_USE" constant.

  The constants marked with "User changeable" may be
  changed based on the LED project is configured.

  The code in this program is pretty much all original.
  I did look at Hover1's 7-segment MAX7219 code. Which
  helped get me started.
 
}}
{
  ******* Private Notes *******

   131225c Displays a message one character at a time.
   25c Appears to be working.
   25d Change font with each message.
   25e Try to rotate text.
   25e Didn't work.
   25g Rotation worked.
   25h Try to add scrolling.
   25j Working reasonably well.
   25k Seems to scroll single rotated array correctly.
   25l Try adding additional array.
   25l Doesn't work. The two displays are almost the
   same. Both have lots of glitches.
   25m Better but a lot of noise on the last display.
   25n Much worse.
   25o Abandon 25n.
   26a Start moving MAX7219 specific code to a
   child object.
   26a Appears to work correctly.
   Object name is now "Max7219MultiArray131226b".
   26b Appears to work correctly using the parent
   object "Max7219Demo131226b".
   26c Start adding a few features.
   
}
CON ' Child Constants

  'CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq ' trick used in "Clock.spin" object.
  'ONE_MS = CLK_FREQ / 1000
  
  
  DEFAULT_SCROLL_INTERVAL = 100 '' User changeable
  LINES_BETWEEN_CHARACTERS = 3  '' User changeable
  SPACE = 32
  FIRST_CHARACTER = SPACE
  LAST_CHARACTER = 126
  LAST_CHARACTER_INDEX = LAST_CHARACTER - FIRST_CHARACTER
  CHARACTERS = LAST_CHARACTER_INDEX + 1
  
CON
'' MAX7219 register names

  ROW_0 = 1
  ROW_1 = 2
  ROW_2 = 3
  ROW_3 = 4
  ROW_4 = 5
  ROW_5 = 6            
  ROW_6 = 7         
  ROW_7 = 8             
   
  DECODE_MODE = 9               ' set to zero for 8x8 array                     
  BRIGHTNESS = 10               ' brightness register 0 - 31
  SCAN_LIMIT = 11               ' limit number or rows scanned set to zero                                   
  SHUT_DOWN = 12                ' set to zero to turn off display
  DISPLAY_TEST = 15             ' set to non-zero to turn on all LEDs

  MIN_BRIGHTNESS = 0
  MAX_BRIGHTNESS = 31
  HALF_BRIGHTNESS = MAX_BRIGHTNESS / 2
  DEFAULT_BRIGHTNESS = HALF_BRIGHTNESS  '' User changeable
   
  
  MAX_ARRAY_ALLOWED = 4         '' User changeable
  LINES_IN_ARRAY = 8
  COLUMNS_IN_ARRAY = LINES_IN_ARRAY
  MAX_ALLOWED_LINES_IN_DISPLAY = LINES_IN_ARRAY * MAX_ARRAY_ALLOWED
 
  
  'activeFontId
  #0, FREE_DESIGN, ASCII_0508

  MAX_FONT_INDEX = ASCII_0508
  FONTS_IN_USE = MAX_FONT_INDEX + 1


VAR

  long arraysInUse, linesInDisplay, bytesToMove
  long arrayAddress
  long nextScrollTimer, scrollInterval
  long fontAddress[FONTS_IN_USE]
  long maxArrayIndex, oneMs

  byte characterWidth[FONTS_IN_USE * CHARACTERS]
  byte activeFontId
  byte clockPin, dataPin, csPin

PUB Init(clockPin_, dataPin_, csPin_, arraysInUse_, arrayAddress_) | row, column, fontPtr

  clockPin := clockPin_
  dataPin := dataPin_
  csPin := csPin_
  arraysInUse := 1 #> arraysInUse_ <#  MAX_ARRAY_ALLOWED
  arrayAddress := arrayAddress_
  linesInDisplay := arraysInUse * LINES_IN_ARRAY
  bytesToMove := linesInDisplay - 1
  maxArrayIndex := arraysInUse - 1
  oneMs := clkfreq / 1000
  scrollInterval := DEFAULT_SCROLL_INTERVAL * oneMs
  activeFontId := FREE_DESIGN
  fontAddress[FREE_DESIGN] := @freeDesign032
  fontAddress[ASCII_0508] := @ASCII_0508_032

  dira[csPin] := 1
  dira[dataPin] := 1
  dira[clockPin] := 1
  outa[csPin] := 1             '' active low                                    

  repeat result from ROW_0 to SHUT_DOWN ' set defaults for 8x8 array
    ShiftOut(result, defaultRegisters[result])

  repeat maxArrayIndex  ' cycle last writes through all arrays.
    ShiftOut(0, 0)
 

  '' Find the width of each character and store the value
  '' in the "characterWidth" array. This allows text
  '' which is scrolled to be appropriately spaced.
  '' It may be a good idea to store these values to EEPROM
  '' so they don't have to be calculated each time
  '' the program starts.
           
  repeat result from 0 to MAX_FONT_INDEX         
    FindWidth(@characterWidth + (result * CHARACTERS), fontAddress[result], CHARACTERS)  
    fontPtr := fontAddress[result]
    repeat CHARACTERS
      RotateCharacter(fontPtr, fontPtr)
      fontPtr += 8
         
PUB SetFont(newFont)

  activeFontId := 0 #> newFont <# MAX_FONT_INDEX
  
PUB SetBrightness(newBrightness)

  newBrightness := MIN_BRIGHTNESS #> newBrightness <# MAX_BRIGHTNESS
  ShiftOut(BRIGHTNESS, newBrightness)
  repeat arraysInUse
    ShiftOut(0, 0)
    
PUB ScrollRotatedText(messagePtr)

  nextScrollTimer := cnt + scrollInterval 
  repeat
    ScrollRotatedCharacter(messagePtr++)  
  while byte[messagePtr]
  
PRI ScrollRotatedCharacter(characterPtr) | fontPtr, localSize   

  fontPtr := fontAddress[activeFontId] + (LINES_IN_ARRAY * (byte[characterPtr] - FIRST_CHARACTER))
  localSize := characterWidth[(activeFontId * CHARACTERS) + (byte[characterPtr] - FIRST_CHARACTER)]
  repeat localSize
    AddRotatedLine(byte[fontPtr++])
  ScrollRotatedBlank(LINES_BETWEEN_CHARACTERS)

PRI ScrollRotatedBlank(newLines)

  repeat newLines
    AddRotatedLine(0)
  
PRI AddRotatedLine(newLine)

  bytemove(arrayAddress, arrayAddress + 1, bytesToMove)
  byte[arrayAddress + bytesToMove] := newLine
  LoadBitsToArray(arrayAddress)
  NextScrollTime
  
PRI NextScrollTime
  
  waitcnt(nextScrollTimer)
  nextScrollTimer += scrollInterval
  
{PUB DisplayText(localPtr) | fontPtr

  Pst.Str(string(13, " DisplayText(", QUOTE))
  Pst.Dec(localPtr)
  Pst.Char(")")          
  repeat
    Pst.Str(string(13, " byte["))
    Pst.Dec(localPtr)
    Pst.Str(string("] = ", QUOTE))
    Pst.Char(byte[localPtr])
    Pst.Char(QUOTE)
    fontPtr := fontAddress[activeFontId] + (8 * (byte[localPtr++] - FIRST_CHARACTER))
    LoadBitsToArray(fontPtr)
    waitcnt(clkfreq / 2 + cnt) 
  while byte[localPtr]
}
PRI FindWidth(destination, source, localSize) '| characterIndex, bitIndex
'' This method should be called before rotating the font.
'' The font being measured should 8x8
'' The font should be left justified.

  result := destination
  repeat localSize
    byte[destination++] := FindCharacterWidth(source)
    'Pst.Str(string(13, " width of ", QUOTE))
    'Pst.Char(((destination - result) - 1) + FIRST_CHARACTER)         
    'Pst.Str(string(QUOTE, " = "))
    'Pst.Dec(byte[destination - 1])         
    source += LINES_IN_ARRAY

  byte[result + (SPACE - FIRST_CHARACTER)] := byte[result + ("n" - FIRST_CHARACTER)] - LINES_BETWEEN_CHARACTERS
  
PRI FindCharacterWidth(characterPtr) | rowIndex, bitIndex

  repeat rowIndex from 0 to 7
    outb[7..0] := byte[characterPtr][rowIndex]
    'Pst.Str(string(13, " %"))
    'Pst.Bin(byte[characterPtr][rowIndex], COLUMNS_IN_ARRAY)
    repeat bitIndex from 0 to 7
      if outb[7 - bitIndex]
        if bitIndex + 1 > result
          result := bitIndex + 1 
  
PUB LoadBitsToArray(localPtr) : row | arrayIndex

  repeat row from ROW_0 to ROW_7
    outa[csPin] := 0
    repeat arrayIndex from 0 to maxArrayIndex
      ClockOut(row, byte[localPtr + (arrayIndex * LINES_IN_ARRAY)])
    outa[csPin] := 1  
    localPtr++    

PUB ShiftOut(address, data)
'' When using multiple arrays, any identical data
'' which is to be sent to all arrays, may be sent
'' with this method. Once the final piece of data
'' has been sent, this method should be called
'' arraysInUse - 1 times with ShiftOut(0, 0).
'' The sequential calling with ShiftOut(0, 0)
'' will make sure each MAX7219 chip gets
'' identical information.

  outa[csPin] := 0
  ClockOut(address, data)
  outa[csPin] := 1

PUB ShiftOutSingle(arrayIndex, address, data)
'' When using multiple arrays, data which
'' is to be sent to a single array will need
'' to have blank data sent before and/or
'' behind it. For example if there were
'' three arrays in use and the brightness of
'' the center arrays was to be set to zero
'' while leaving the two other arrays at
'' the original brightness level, then the
'' call of ShiftOutSingle(1, BRIGHTNESS, 0)
'' would cause blank data to be sent both
'' before and after the brightness setting
'' command.

  outa[csPin] := 0
  result := (arraysInUse - 1) - arrayIndex
  if result > 0
    repeat result
      ClockOut(0, 0)
  ClockOut(address, data)
  if arrayIndex
    repeat arrayIndex
      ClockOut(0, 0)
  outa[csPin] := 1

PUB ClockOut(address, data)

  result := (address << 8) + data
  result <<= 16
  repeat 16
    result <-= 1
    outa[dataPin] := result
    outa[clockPin] := 1
    'Delay 
    
    outa[clockPin] := 0
   
{PRI DebugLoop | address, data

  repeat

    Pst.Home
    Pst.ClearEnd
    Pst.NewLine
    Pst.Str(string(" Please enter address to update on the MAX2719 chip. "))
    Pst.ClearEnd
    Pst.NewLine
    Pst.ClearBelow     
    Pst.NewLine
    Pst.ClearEnd
    address := Pst.DecIn
    Pst.NewLine
    Pst.Str(string(" You entered ", QUOTE))
    Pst.Dec(address)
    Pst.Str(string(QUOTE, " or "))
    ReadableBin(address)
    Pst.ClearEnd
    Pst.NewLine
    Pst.Str(string(" Please enter number to write to address #"))
    Pst.Dec(address) 
    Pst.Str(string(" on MAX2719 chip. "))
    Pst.ClearEnd
    Pst.NewLine
    Pst.Str(string(" (Or enter a negative number to abort write.)"))
    Pst.ClearEnd    
    Pst.NewLine
    Pst.ClearEnd
    result := Pst.DecIn
    Pst.NewLine
    Pst.Str(string(" You entered ", QUOTE))
    Pst.Dec(result)
    Pst.Str(string(QUOTE, " or "))
    ReadableBin(result)
    Pst.ClearEnd
    if result => 0
      ShiftOut(address, result)
    else
      Pst.NewLine
      Pst.Str(string(" Write command to MAX7219 aborted."))
      Pst.ClearEnd  
    Pst.NewLine
    Pst.ClearEnd
    Pst.ClearBelow
    waitcnt(clkfreq * 2 + cnt)       
   }
PUB GetObjectName

  result := @objectName

PRI RotateCharacter(destination, source) | temp[2], scratch[2], rowOld, rowNew, colOld, colNew, testResult, testBit, bitOfInterest

  'Pst.Str(string(13, " RotateCharacter"))
    
  longfill(@temp, 0, 2)
  bytemove(@scratch, source, LINES_IN_ARRAY) 
  repeat rowNew from 0 to 7
    bitOfInterest := 7 - rowNew
    repeat rowOld from 0 to 7
      dirb[7..0] := byte[@scratch][rowOld]
      outb[rowOld] := dirb[bitOfInterest]
    byte[@temp][rowNew] := outb
    'Pst.Str(string(13, " %"))
    'Pst.Bin(byte[@temp][rowNew], LINES_IN_ARRAY)
              
  bytemove(destination, @temp, LINES_IN_ARRAY)
  
DAT
                                   
defaultRegisters        byte 0[10], DEFAULT_BRIGHTNESS, 255, 1
                                  
DAT freeDesign032       byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign033           byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %00000000
                        byte %11000000
              
freeDesign034           byte %10100000
                        byte %10100000
                        byte %10100000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                         
freeDesign035           byte %01010000
                        byte %11111000
                        byte %01010000
                        byte %11111000
                        byte %01010000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign036           byte %01000000
                        byte %01110000
                        byte %11000000
                        byte %11100000
                        byte %01110000
                        byte %00110000
                        byte %11100000
                        byte %01000000
              
freeDesign037           byte %01111100
                        byte %10100100
                        byte %10100100
                        byte %01001000
                        byte %00010010
                        byte %00100101
                        byte %00100101
                        byte %00100010
              
freeDesign038           byte %00111100
                        byte %01100100
                        byte %01100000
                        byte %00110000
                        byte %01000110
                        byte %11001100
                        byte %11001100
                        byte %01110110
              
freeDesign039           byte %10000000
                        byte %10000000
                        byte %10000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign040           byte %00100000
                        byte %01000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %01000000
                        byte %00100000

freeDesign041           byte %10000000
                        byte %01000000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01000000
                        byte %10000000
                         
freeDesign042           byte %00000000
                        byte %00000000
                        byte %00100000
                        byte %10101000
                        byte %01110000
                        byte %10101000
                        byte %00100000
                        byte %00000000

freeDesign043           byte %00000000
                        byte %00000000
                        byte %00100000
                        byte %00100000
                        byte %11111000
                        byte %00100000
                        byte %00100000
                        byte %00000000

freeDesign044           byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %01000000
                        byte %10000000
              
freeDesign045           byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11100000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
              
freeDesign046           byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %11000000
              
freeDesign047           byte %00010000
                        byte %00010000
                        byte %00100000
                        byte %00100000
                        byte %01000000
                        byte %01000000
                        byte %10000000
                        byte %10000000
              
freeDesign048           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11101100
                        byte %11011100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign049           byte %01100000
                        byte %11100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
              
freeDesign050           byte %01110000
                        byte %10011000
                        byte %00011000
                        byte %00011000
                        byte %00110000
                        byte %01100000
                        byte %11000000
                        byte %11111000
              
freeDesign051           byte %01110000
                        byte %10011000
                        byte %00011000
                        byte %01110000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %11110000
              
freeDesign052           byte %00011000
                        byte %00011000
                        byte %01011000
                        byte %01011000
                        byte %10011000
                        byte %11111100
                        byte %00011000
                        byte %00011000
              
freeDesign053           byte %11111000
                        byte %11000000
                        byte %11000000
                        byte %11110000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %11110000
              
freeDesign054           byte %00111000
                        byte %01000000
                        byte %11000000
                        byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign055           byte %11111100
                        byte %00001100
                        byte %00011000
                        byte %00011000
                        byte %00110000
                        byte %00110000
                        byte %01100000
                        byte %01100000
              
freeDesign056           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign057           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111100
                        byte %00001100
                        byte %00011000
                        byte %01110000
              
freeDesign058           byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %11000000
                        byte %00000000
                        byte %11000000
                        byte %11000000
                        byte %00000000

freeDesign059           byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %11000000
                        byte %00000000
                        byte %11000000
                        byte %01000000
                        byte %10000000
                         
freeDesign060           byte %00000000
                        byte %00110000
                        byte %01100000
                        byte %11000000
                        byte %11000000
                        byte %01100000
                        byte %00110000
                        byte %00000000
              
freeDesign061           byte %00000000
                        byte %00000000
                        byte %11100000
                        byte %00000000
                        byte %11100000
                        byte %00000000
                        byte %00000000
                        byte %00000000
              
freeDesign062           byte %00000000
                        byte %11000000
                        byte %01100000
                        byte %00110000
                        byte %00110000
                        byte %01100000
                        byte %11000000
                        byte %00000000
              
freeDesign063           byte %11111000
                        byte %11001100
                        byte %00001100
                        byte %00011000
                        byte %00110000
                        byte %00100000
                        byte %00000000
                        byte %00110000
              
freeDesign064           byte %00111110
                        byte %01000001
                        byte %10011101
                        byte %10100101
                        byte %10100101
                        byte %10111111
                        byte %01000000
                        byte %00111110
              
freeDesign065           byte %00110000
                        byte %00110000
                        byte %00111000
                        byte %01011000
                        byte %01011000
                        byte %11111100
                        byte %10001100
                        byte %10001100
              
freeDesign066           byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
              
freeDesign067           byte %01111000
                        byte %11001000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11001000
                        byte %01110000
              
freeDesign068           byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
              
freeDesign069           byte %11111000
                        byte %11000000
                        byte %11000000
                        byte %11110000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11111000
                         
freeDesign070           byte %11111000
                        byte %11000000
                        byte %11000000
                        byte %11110000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
              
freeDesign071           byte %01111100
                        byte %11000100
                        byte %11000000
                        byte %11000000
                        byte %11011100
                        byte %11001100
                        byte %11001100
                        byte %01110100
                         
freeDesign072           byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
              
freeDesign073           byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
              
freeDesign074           byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %00011000
                        byte %10011000
                        byte %01110000
              
freeDesign075           byte %11000100
                        byte %11001000
                        byte %11010000
                        byte %11110000
                        byte %11011000
                        byte %11011000
                        byte %11001100
                        byte %11001100
              
freeDesign076           byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11111000
                         
freeDesign077           byte %11000110
                        byte %11000110
                        byte %11101110
                        byte %11101110
                        byte %10110110
                        byte %10110110
                        byte %10000110
                        byte %10000110
              
freeDesign078           byte %11001000 
                        byte %11001000
                        byte %11101000
                        byte %11101000
                        byte %10111000
                        byte %10111000
                        byte %10011000
                        byte %10011000
              
freeDesign079           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign080           byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                         
freeDesign081           byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
                        byte %00001110
                         
freeDesign082           byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
              
freeDesign083           byte %01111000
                        byte %11001000
                        byte %11100000
                        byte %01110000
                        byte %00111000
                        byte %00011000
                        byte %10011000
                        byte %11110000
              
freeDesign084           byte %11111100
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
              
freeDesign085           byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
              
freeDesign086           byte %11000100
                        byte %11000100
                        byte %01101000
                        byte %01101000
                        byte %01101000
                        byte %00110000
                        byte %00110000
                        byte %00110000
              
freeDesign087           byte %11000001
                        byte %11000001
                        byte %01101001
                        byte %01101101
                        byte %01101101
                        byte %00110011
                        byte %00110011
                        byte %00110011
              
freeDesign088           byte %11001000
                        byte %11001000
                        byte %01110000
                        byte %01100000
                        byte %00110000
                        byte %01110000
                        byte %10011000
                        byte %10011000
              
freeDesign089           byte %11000100
                        byte %11000100
                        byte %01101000
                        byte %01101000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
              
freeDesign090           byte %11111000
                        byte %00011000
                        byte %00110000
                        byte %00110000
                        byte %01100000
                        byte %01100000
                        byte %11000000
                        byte %11111000
              
freeDesign091           byte %11110000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11110000

freeDesign092           byte %00000000
                        byte %11000000
                        byte %01100000
                        byte %01100000
                        byte %00110000
                        byte %00110000
                        byte %00011000
                        byte %00000000

freeDesign093           byte %11110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %00110000
                        byte %11110000
                         
freeDesign094           byte %00000000
                        byte %00000000
                        byte %00100000
                        byte %01010000
                        byte %10001000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign095           byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %00000000
                        byte %11111111
                         
freeDesign096           byte %00000000
                        byte %00000000
                        byte %11000000
                        byte %01100000
                        byte %00110000
                        byte %00000000
                        byte %00000000
                        byte %00000000

freeDesign097           byte %00000000
                        byte %01111000
                        byte %00001100
                        byte %01111100
                        byte %11001100
                        byte %11001100
                        byte %01111100
                        byte %00000000
                         
freeDesign098           byte %11000000
                        byte %11000000
                        byte %11111000
                        byte %11001100 
                        byte %11001100 
                        byte %11001100
                        byte %11111000
                        byte %00000000

freeDesign099           byte %00000000
                        byte %01111000
                        byte %11001000
                        byte %11000000
                        byte %11000000
                        byte %11001000
                        byte %01110000
                        byte %00000000

freeDesign100           byte %00001100
                        byte %00001100
                        byte %01111100
                        byte %11001100 
                        byte %11001100 
                        byte %11001100 
                        byte %01111100
                        byte %00000000

freeDesign101           byte %00000000
                        byte %01111000
                        byte %11001100
                        byte %11011000
                        byte %11100000
                        byte %11000100
                        byte %01111000
                        byte %00000000
              
freeDesign102           byte %00111000
                        byte %01100000
                        byte %11110000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %00000000
                         
freeDesign103           byte %00000000
                        byte %11111110
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111100
                        byte %00001100
                        byte %01111000
                         
freeDesign104           byte %11000000
                        byte %11000000
                        byte %11011000
                        byte %11101100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %00000000

freeDesign105           byte %11000000 
                        byte %00000000
                        byte %11000000 
                        byte %11000000 
                        byte %11000000 
                        byte %11000000 
                        byte %11000000
                        byte %00000000

freeDesign106           byte %01100000 
                        byte %00000000
                        byte %01100000 
                        byte %01100000 
                        byte %01100000 
                        byte %01100000
                        byte %01100000
                        byte %11000000

freeDesign107           byte %11000000 
                        byte %11000100 
                        byte %11001000 
                        byte %11010000 
                        byte %11110000 
                        byte %11011000 
                        byte %11001100
                        byte %00000000

freeDesign108           byte %11000000 
                        byte %11000000
                        byte %11000000 
                        byte %11000000 
                        byte %11000000 
                        byte %11000000 
                        byte %11000000
                        byte %00000000

freeDesign109           byte %00000000
                        byte %11010010
                        byte %11111111
                        byte %11011011
                        byte %11011011
                        byte %11011011
                        byte %11011011
                        byte %00000000

freeDesign110           byte %00000000
                        byte %11011000
                        byte %11101100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %00000000

freeDesign111           byte %00000000
                        byte %01111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
                        byte %00000000

freeDesign112           byte %00000000
                        byte %11111000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11111000
                        byte %11000000
                        byte %11000000
                         
freeDesign113           byte %00000000
                        byte %01111100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111100
                        byte %00001100
                        byte %00001100
              
freeDesign114           byte %00000000
                        byte %11010000
                        byte %11110000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %00000000

freeDesign115           byte %00000000
                        byte %01110000
                        byte %11000000
                        byte %11100000
                        byte %01110000
                        byte %00110000
                        byte %11100000
                        byte %00000000

freeDesign116           byte %01100000
                        byte %01100000
                        byte %11110000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
                        byte %01100000
              
freeDesign117           byte %00000000
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %11001100
                        byte %01111000
                        byte %00000000
                         
freeDesign118           byte %00000000
                        byte %11001100
                        byte %11001100
                        byte %01101000
                        byte %01101000
                        byte %00110000
                        byte %00110000
                        byte %00000000

freeDesign119           byte %00000000
                        byte %11011011
                        byte %11011011
                        byte %11011011
                        byte %11011011
                        byte %01101100
                        byte %01101100
                        byte %00000000
              
freeDesign120           byte %00000000
                        byte %11001100
                        byte %11101000
                        byte %01110000
                        byte %00111000
                        byte %01011100
                        byte %11001100
                        byte %00000000

freeDesign121           byte %00000000
                        byte %11001100
                        byte %11001100
                        byte %01101000
                        byte %01101000
                        byte %00110000
                        byte %00010000
                        byte %01110000
              
freeDesign122           byte %00000000
                        byte %11110000
                        byte %00110000
                        byte %01100000
                        byte %01100000
                        byte %11000000
                        byte %11110000
                        byte %00000000
              
freeDesign123           byte %00111000
                        byte %01100000
                        byte %01100000
                        byte %11000000
                        byte %11000000
                        byte %01100000
                        byte %01100000
                        byte %00111000
                         
freeDesign124           byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
                        byte %11000000
              
freeDesign125           byte %11100000
                        byte %00110000
                        byte %00110000
                        byte %00011000
                        byte %00011000
                        byte %00110000
                        byte %00110000
                        byte %11100000
              
freeDesign126           byte %00000000
                        byte %00000000
                        byte %01110000
                        byte %10011001
                        byte %00001110
                        byte %00000000
                        byte %00000000
                        byte %00000000

DAT
'--------------------------------- [ ASCII ]--------------------------------------
'----------------------------------[  5x8  ]--------------------------------------
DAT ASCII_0508_032 byte    $00,$00,$00,$00,$00,$00,$00,$00    ' - -           
ASCII_0508_033 byte        $20,$20,$20,$20,$20,$00,$20,$00    ' -!-           
ASCII_0508_034 byte        $50,$50,$50,$00,$00,$00,$00,$00    ' -"-           
ASCII_0508_035 byte        $50,$50,$F8,$50,$F8,$50,$50,$00    ' -#-           
ASCII_0508_036 byte        $20,$78,$C0,$70,$28,$F0,$20,$00    ' -$-           
ASCII_0508_037 byte        $C0,$C8,$10,$20,$40,$98,$18,$00    ' -%-           
ASCII_0508_038 byte        $40,$A0,$A0,$40,$A8,$90,$68,$00    ' -&-           
ASCII_0508_039 byte        $30,$20,$40,$00,$00,$00,$00,$00    ' -'-           
ASCII_0508_040 byte        $10,$20,$40,$40,$40,$20,$10,$00    ' -(-           
ASCII_0508_041 byte        $40,$20,$10,$10,$10,$20,$40,$00    ' -)-           
ASCII_0508_042 byte        $20,$A8,$70,$20,$70,$A8,$20,$00    ' -*-           
ASCII_0508_043 byte        $20,$20,$20,$F8,$20,$20,$20,$00    ' -+-           
ASCII_0508_044 byte        $00,$00,$00,$00,$60,$40,$80,$00    ' -,-           
ASCII_0508_045 byte        $00,$00,$00,$F8,$00,$00,$00,$00    ' ---           
ASCII_0508_046 byte        $00,$00,$00,$00,$00,$60,$60,$00    ' -.-           
ASCII_0508_047 byte        $00,$08,$10,$20,$40,$80,$00,$00    ' -/-           
ASCII_0508_048 byte        $70,$88,$98,$A8,$C8,$88,$70,$00    ' -0-           
ASCII_0508_049 byte        $20,$60,$20,$20,$20,$20,$70,$00    ' -1-           
ASCII_0508_050 byte        $70,$88,$08,$30,$40,$80,$F8,$00    ' -2-           
ASCII_0508_051 byte        $F8,$08,$10,$30,$08,$88,$70,$00    ' -3-           
ASCII_0508_052 byte        $10,$30,$50,$90,$F8,$10,$10,$00    ' -4-           
ASCII_0508_053 byte        $F8,$80,$F0,$08,$08,$88,$70,$00    ' -5-           
ASCII_0508_054 byte        $38,$40,$80,$F0,$88,$88,$70,$00    ' -6-           
ASCII_0508_055 byte        $F8,$08,$10,$20,$40,$40,$40,$00    ' -7-           
ASCII_0508_056 byte        $70,$88,$88,$70,$88,$88,$70,$00    ' -8-           
ASCII_0508_057 byte        $70,$88,$88,$78,$08,$10,$E0,$00    ' -9-           
ASCII_0508_058 byte        $00,$60,$60,$00,$60,$60,$00,$00    ' -:-           
ASCII_0508_059 byte        $00,$60,$60,$00,$60,$60,$80,$00    ' -;-           
ASCII_0508_060 byte        $10,$20,$40,$80,$40,$20,$10,$00    ' -<-           
ASCII_0508_061 byte        $00,$00,$F8,$00,$F8,$00,$00,$00    ' -=-           
ASCII_0508_062 byte        $40,$20,$10,$08,$10,$20,$40,$00    ' ->-           
ASCII_0508_063 byte        $70,$88,$10,$20,$20,$00,$20,$00    ' -?-           
ASCII_0508_064 byte        $70,$88,$B8,$A8,$B8,$80,$78,$00    ' -@-           
ASCII_0508_065 byte        $20,$50,$88,$88,$F8,$88,$88,$00    ' -A-           
ASCII_0508_066 byte        $F0,$88,$88,$F0,$88,$88,$F0,$00    ' -B-           
ASCII_0508_067 byte        $70,$88,$80,$80,$80,$88,$70,$00    ' -C-           
ASCII_0508_068 byte        $F0,$88,$88,$88,$88,$88,$F0,$00    ' -D-           
ASCII_0508_069 byte        $F8,$80,$80,$F0,$80,$80,$F8,$00    ' -E-           
ASCII_0508_070 byte        $F8,$80,$80,$F0,$80,$80,$80,$00    ' -F-           
ASCII_0508_071 byte        $70,$88,$80,$80,$B8,$88,$78,$00    ' -G-           
ASCII_0508_072 byte        $88,$88,$88,$F8,$88,$88,$88,$00    ' -H-           
ASCII_0508_073 byte        $70,$20,$20,$20,$20,$20,$70,$00    ' -I-           
ASCII_0508_074 byte        $38,$10,$10,$10,$10,$90,$60,$00    ' -J-           
ASCII_0508_075 byte        $88,$90,$A0,$C0,$A0,$90,$88,$00    ' -K-           
ASCII_0508_076 byte        $80,$80,$80,$80,$80,$80,$F8,$00    ' -L-           
ASCII_0508_077 byte        $88,$D8,$A8,$A8,$88,$88,$88,$00    ' -M-           
ASCII_0508_078 byte        $88,$88,$C8,$A8,$98,$88,$88,$00    ' -N-           
ASCII_0508_079 byte        $70,$88,$88,$88,$88,$88,$70,$00    ' -O-           
ASCII_0508_080 byte        $F0,$88,$88,$F0,$80,$80,$80,$00    ' -P-           
ASCII_0508_081 byte        $70,$88,$88,$88,$A8,$90,$68,$00    ' -Q-           
ASCII_0508_082 byte        $F0,$88,$88,$F0,$A0,$90,$88,$00    ' -R-           
ASCII_0508_083 byte        $70,$88,$80,$70,$08,$88,$70,$00    ' -S-           
ASCII_0508_084 byte        $F8,$20,$20,$20,$20,$20,$20,$00    ' -T-           
ASCII_0508_085 byte        $88,$88,$88,$88,$88,$88,$70,$00    ' -U-           
ASCII_0508_086 byte        $88,$88,$88,$88,$88,$50,$20,$00    ' -V-           
ASCII_0508_087 byte        $88,$88,$88,$A8,$A8,$D8,$88,$00    ' -W-           
ASCII_0508_088 byte        $88,$88,$50,$20,$50,$88,$88,$00    ' -X-           
ASCII_0508_089 byte        $88,$88,$50,$20,$20,$20,$20,$00    ' -Y-           
ASCII_0508_090 byte        $F8,$08,$10,$20,$40,$80,$F8,$00    ' -Z-           
ASCII_0508_091 byte        $F0,$C0,$C0,$C0,$C0,$C0,$F0,$00    ' -[-           
ASCII_0508_092 byte        $00,$80,$40,$20,$10,$08,$00,$00    ' -\-           
ASCII_0508_093 byte        $78,$18,$18,$18,$18,$18,$78,$00    ' -]-           
ASCII_0508_094 byte        $20,$70,$A8,$20,$20,$20,$20,$00    ' -^-           
ASCII_0508_095 byte        $00,$20,$40,$F8,$40,$20,$00,$00    ' -_-           
ASCII_0508_096 byte        $20,$10,$08,$00,$00,$00,$00,$00    ' -`-           
ASCII_0508_097 byte        $00,$00,$E0,$10,$70,$90,$68,$00    ' -a-           
ASCII_0508_098 byte        $80,$80,$B0,$C8,$88,$C8,$B0,$00    ' -b-           
ASCII_0508_099 byte        $00,$00,$70,$88,$80,$80,$70,$00    ' -c-           
ASCII_0508_100 byte        $08,$08,$68,$98,$88,$98,$68,$00    ' -d-           
ASCII_0508_101 byte        $00,$00,$70,$88,$F0,$80,$70,$00    ' -e-           
ASCII_0508_102 byte        $30,$48,$40,$F0,$40,$40,$40,$00    ' -f-           
ASCII_0508_103 byte        $00,$00,$70,$88,$88,$78,$08,$F0    ' -g-           
ASCII_0508_104 byte        $80,$80,$B0,$C8,$88,$88,$88,$00    ' -h-           
ASCII_0508_105 byte        $20,$00,$00,$20,$20,$20,$20,$00    ' -i-           
ASCII_0508_106 byte        $10,$00,$00,$30,$10,$10,$10,$60    ' -j-           
ASCII_0508_107 byte        $80,$80,$90,$A0,$C0,$A0,$98,$00    ' -k-           
ASCII_0508_108 byte        $60,$20,$20,$20,$20,$20,$70,$00    ' -l-           
ASCII_0508_109 byte        $00,$00,$50,$A8,$A8,$A8,$A8,$00    ' -m-           
ASCII_0508_110 byte        $00,$00,$B0,$48,$48,$48,$48,$00    ' -n-           
ASCII_0508_111 byte        $00,$00,$70,$88,$88,$88,$70,$00    ' -o-           
ASCII_0508_112 byte        $00,$00,$F0,$88,$88,$F0,$80,$80    ' -p-           
ASCII_0508_113 byte        $00,$00,$78,$88,$88,$78,$08,$08    ' -q-           
ASCII_0508_114 byte        $00,$00,$B0,$48,$40,$40,$40,$00    ' -r-           
ASCII_0508_115 byte        $00,$00,$78,$80,$70,$08,$F0,$00    ' -s-           
ASCII_0508_116 byte        $40,$40,$F8,$40,$40,$48,$30,$00    ' -t-           
ASCII_0508_117 byte        $00,$00,$90,$90,$90,$90,$68,$00    ' -u-           
ASCII_0508_118 byte        $00,$00,$88,$88,$88,$50,$20,$00    ' -v-           
ASCII_0508_119 byte        $00,$00,$A8,$A8,$A8,$A8,$50,$00    ' -w-           
ASCII_0508_120 byte        $00,$00,$88,$50,$20,$50,$88,$00    ' -x-           
ASCII_0508_121 byte        $00,$00,$88,$88,$98,$68,$08,$F0    ' -y-           
ASCII_0508_122 byte        $00,$00,$F8,$10,$20,$40,$F8,$00    ' -z-           
ASCII_0508_123 byte        $20,$40,$40,$80,$40,$40,$20,$00    ' -{-           
ASCII_0508_124 byte        $20,$20,$20,$00,$20,$20,$20,$00    ' -|-           
ASCII_0508_125 byte        $20,$10,$10,$08,$10,$10,$20,$00    ' -}-           
ASCII_0508_126 byte        $00,$00,$40,$A8,$10,$00,$00,$00    ' -~-           
ASCII_0508_127 byte        $A8,$50,$A8,$50,$A8,$50,$A8,$00    ' --
      